// react:
import { 
// react:
default as React, 
// hooks:
useRef, } from 'react';
// cssfn:
import { 
// checks if a certain css feature is supported by the running browser:
supportsHasPseudoClass, } from '@cssfn/core'; // writes css in javascript
import { 
// style sheets:
dynamicStyleSheet, } from '@cssfn/cssfn-react'; // writes css in react hook
// reusable-ui core:
import { 
// react helper hooks:
useEvent, useMergeEvents, useMergeRefs, useMergeClasses, 
// an accessibility management system:
usePropEnabled, usePropReadOnly, useUncontrollableActivatable, } from '@reusable-ui/core'; // a set of reusable-ui packages which are responsible for building any component
// reusable-ui components:
import { EditableActionControl, } from '@reusable-ui/editable-action-control'; // a base component
// internals:
import { useCheckVariant, } from './variants/CheckVariant.js';
// styles:
export const useCheckStyleSheet = dynamicStyleSheet(() => import(/* webpackPrefetch: true */ './styles/styles.js'), {
    id: 'nx58strmq2',
    lazyCsr: supportsHasPseudoClass(), // dealing with browsers that don't support the :has() selector
});
// handlers:
const handleChangeDummy = (_event) => {
    /* nothing to do */
};
export const handleInputClickTriggersChange = (event) => {
    event.stopPropagation(); // a hack to prevent the `triggerChange` triggers `onClick` => re-trigger `triggerChange` => infinity trigger
};
const Check = (props) => {
    // styles:
    const styleSheet = useCheckStyleSheet();
    // variants:
    const checkVariant = useCheckVariant(props);
    // rest props:
    const { 
    // refs:
    elmRef, 
    // variants:
    checkStyle: checkStyle, // use
    // states:
    defaultActive, // take, to be handled by `useUncontrollableActivatable`
    active, // take, to be handled by `useUncontrollableActivatable`
    inheritActive, // take, to be handled by `useUncontrollableActivatable`
    onActiveChange, // take, to be handled by `useUncontrollableActivatable`
    // accessibilities:
    // still on <EditableActionControl> element
    // autoFocus,
    // tabIndex,
    // enterKeyHint,
    label, 
    // validations:
    required, 
    // formats:
    type = 'checkbox', 
    // forms:
    name, form, 
    // values:
    defaultValue, value, onChange, // forwards to `input[type]`
    defaultChecked, // take, to be aliased to `defaultActive`
    checked, // take, to be aliased to `active`
    // children:
    children, ...restEditableActionControlProps } = props;
    // refs:
    const inputRefInternal = useRef(null);
    const mergedInputRef = useMergeRefs(
    // preserves the original `elmRef`:
    elmRef, inputRefInternal);
    // states:
    const [isActive, , toggleActive] = useUncontrollableActivatable({
        enabled: props.enabled,
        inheritEnabled: props.inheritEnabled,
        readOnly: props.readOnly,
        inheritReadOnly: props.inheritReadOnly,
        defaultActive: defaultActive ?? defaultChecked,
        active: active ?? checked,
        inheritActive,
        onActiveChange,
    }, /*changeEventTarget :*/ inputRefInternal);
    // fn props:
    const propEnabled = usePropEnabled(props);
    const propReadOnly = usePropReadOnly(props);
    // classes:
    const variantClasses = useMergeClasses(
    // preserves the original `variantClasses`:
    props.variantClasses, 
    // variants:
    checkVariant.class);
    // handlers:
    const handleClickInternal = useEvent((event) => {
        // conditions:
        if (event.defaultPrevented)
            return; // the event was already handled by user => nothing to do
        // actions:
        toggleActive(); // handle click as toggle [active]
        event.preventDefault(); // handled
    });
    const handleClick = useMergeEvents(
    // preserves the original `onClick`:
    props.onClick, 
    // actions:
    handleClickInternal);
    const handleKeyDownInternal = useEvent((event) => {
        // conditions:
        if (event.defaultPrevented)
            return; // the event was already handled by user => nothing to do
        // actions:
        if ((event.key === ' ') || (event.code === 'Space')) {
            event.preventDefault(); // prevents pressing space for scrolling page
        } // if
    });
    const handleKeyDown = useMergeEvents(
    // preserves the original `onKeyDown`:
    props.onKeyDown, 
    // actions:
    handleKeyDownInternal);
    const handleKeyUpInternal = useEvent((event) => {
        // conditions:
        if (event.defaultPrevented)
            return; // the event was already handled by user => nothing to do
        // actions:
        if ((event.key === ' ') || (event.code === 'Space')) {
            toggleActive(); // handle click as toggle [active]
            event.preventDefault(); // handled
        } // if
    });
    const handleKeyUp = useMergeEvents(
    // preserves the original `onKeyUp`:
    props.onKeyUp, 
    // actions:
    handleKeyUpInternal);
    const handleChange = useMergeEvents(
    // preserves the original `onChange`:
    onChange, 
    // dummy:
    handleChangeDummy);
    // jsx:
    return (React.createElement(EditableActionControl, { ...restEditableActionControlProps, 
        // semantics:
        tag: props.tag ?? 'span', semanticTag: props.semanticTag ?? '', semanticRole: props.semanticRole ?? 'checkbox', "aria-label": props['aria-label'] ?? label, 
        // variants:
        nude: props.nude ?? (((checkStyle === 'button') || (checkStyle === 'toggleButton'))
            ?
                false // filled : <Check checkStyle='button'> | <Check checkStyle='toggleButton'>
            :
                true // nude   : <Check> | <Check checkStyle='switch'>
        ), mild: props.mild ?? false, 
        // states:
        active: isActive, 
        // classes:
        mainClass: props.mainClass ?? styleSheet.main, variantClasses: variantClasses, 
        // handlers:
        onClick: handleClick, onKeyDown: handleKeyDown, onKeyUp: handleKeyUp, ...{
        // actions:
        // type,
        } },
        React.createElement("input", { 
            // refs:
            ref: mergedInputRef, ...{
                // autoFocus,    // still on <EditableControl> element
                tabIndex: -1, // not focusable
                // enterKeyHint, // not supported
            }, disabled: !propEnabled, readOnly: propReadOnly, ...{
                name,
                form,
            }, ...{
                defaultValue,
                value,
                // defaultChecked,       // fully controllable, no defaultChecked
                checked: isActive,
                onChange: handleChange,
            }, ...{
                required,
            }, ...{
                type,
            }, 
            // handlers:
            onClick: handleInputClickTriggersChange }),
        !!children && React.createElement("span", null, children)));
};
export { Check, Check as default, };
