// cssfn:
import { 
// writes css in javascript:
keyframes, style, escapeSvg, 
// reads/writes css variables configuration:
cssConfig, } from '@cssfn/core'; // writes css in javascript
// reusable-ui core:
import { 
// a capability of UI to be checked:
usesCheckable, } from '@reusable-ui/core'; // a set of reusable-ui packages which are responsible for building any component
// configs:
export const [checks, checkValues, cssCheckConfig] = cssConfig(() => {
    // dependencies:
    const { checkableVars: { filterIn: checkFilterIn, filterOut: checkFilterOut, transformIn: checkTransformIn, transformOut: checkTransformOut } } = usesCheckable();
    //#region keyframes
    const frameChecked = style({
        filter: [[
                checkFilterIn,
            ]],
        transform: [[
                checkTransformIn,
            ]],
    });
    const frameCleared = style({
        filter: [[
                checkFilterOut,
            ]],
        transform: [[
                checkTransformOut,
            ]],
    });
    const [keyframesRegularCheckRule, keyframesRegularCheck] = keyframes({
        from: frameCleared,
        to: frameChecked,
    });
    keyframesRegularCheck.value = 'regularCheck'; // the @keyframes name should contain 'check' in order to be recognized by `usesCheckable`
    const [keyframesRegularClearRule, keyframesRegularClear] = keyframes({
        from: frameChecked,
        to: frameCleared,
    });
    keyframesRegularClear.value = 'regularClear'; // the @keyframes name should contain 'clear' in order to be recognized by `usesCheckable`
    const frameChecking = style({
        transformOrigin: 'left',
        transform: [[
                'scaleX(1.2)',
                checkTransformIn,
            ]],
    });
    const frameClearing = style({
        transformOrigin: 'right',
        transform: [[
                'scaleX(1.2)',
                checkTransformOut,
            ]],
    });
    const [keyframesSwitchCheckRule, keyframesSwitchCheck] = keyframes({
        from: frameCleared,
        '50%': frameChecking,
        to: frameChecked,
    });
    keyframesSwitchCheck.value = 'switchCheck'; // the @keyframes name should contain 'check' in order to be recognized by `usesCheckable`
    const [keyframesSwitchClearRule, keyframesSwitchClear] = keyframes({
        from: frameChecked,
        '50%': frameClearing,
        to: frameCleared,
    });
    keyframesSwitchClear.value = 'switchClear'; // the @keyframes name should contain 'clear' in order to be recognized by `usesCheckable`
    //#endregion keyframes
    return {
        // spacings:
        spacing: '0.3em',
        // animations:
        // forked from Bootstrap 5:
        indicator: `url("data:image/svg+xml,${escapeSvg("<svg xmlns='http://www.w3.org/2000/svg' viewBox='0 0 20 20'><path fill='none' stroke='#000' stroke-linecap='round' stroke-linejoin='round' stroke-width='3' d='M6 10l3 3 6-6'/></svg>")}")`,
        switchIndicator: `url("data:image/svg+xml,${escapeSvg("<svg xmlns='http://www.w3.org/2000/svg' viewBox='-4 -4 8 8'><circle r='3' fill='#000'/></svg>")}")`,
        checkFilterIn: [[
                'opacity(100%)',
            ]],
        checkFilterOut: [[
                'opacity(0%)',
            ]],
        checkTransformIn: 'initial',
        checkTransformOut: 'initial',
        ...keyframesRegularCheckRule,
        ...keyframesRegularClearRule,
        checkAnimIn: [
            ['150ms', 'ease-out', 'both', keyframesRegularCheck],
        ],
        checkAnimOut: [
            ['150ms', 'ease-out', 'both', keyframesRegularClear],
        ],
        switchCheckFilterIn: [[
                'opacity(100%)',
            ]],
        switchCheckFilterOut: [[
                'opacity(50%)',
            ]],
        switchCheckTransformIn: [[
                'translateX(0.5em)',
            ]],
        switchCheckTransformOut: [[
                'translateX(-0.5em)',
            ]],
        ...keyframesSwitchCheckRule,
        ...keyframesSwitchClearRule,
        switchCheckAnimIn: [
            ['200ms', 'ease-out', 'both', keyframesSwitchCheck],
        ],
        switchCheckAnimOut: [
            ['200ms', 'ease-out', 'both', keyframesSwitchClear],
        ],
    };
}, { prefix: 'chk' });
