// cssfn:
import { 
// writes css in javascript:
rule, variants, ifNotLastChild, children, style, vars, 
// reads/writes css variables configuration:
usesCssProps, usesPrefixedProps, overwriteProps, 
// writes complex stylesheets in simpler way:
watchChanges, memoizeStyle, } from '@cssfn/core'; // writes css in javascript
// reusable-ui core:
import { 
// a border (stroke) management system:
borderRadiuses, 
// reusable common layouts:
fillTextLineHeightLayout, 
// foreground (text color) stuff of UI:
usesForeground, 
// border (stroke) stuff of UI:
usesBorder, 
// padding (inner spacing) stuff of UI:
usesPadding, 
// size options of UI:
usesResizable, 
// mild (soft color) variant of UI:
usesMildable, 
// nude variant of UI:
ifNotNude, ifNude, 
// a capability of UI to be focused:
usesFocusable, 
// shows the UI as clicked when activated:
usesActiveAsClick, 
// a capability of UI to be checked:
usesCheckable, } from '@reusable-ui/core'; // a set of reusable-ui packages which are responsible for building any component
// reusable-ui components:
import { 
// styles:
onEditableActionControlStylesChange, usesEditableActionControlLayout, usesEditableActionControlVariants, usesEditableActionControlStates, } from '@reusable-ui/editable-action-control'; // a base component
import { 
// styles:
usesIconImage, } from '@reusable-ui/icon'; // an icon component
import { 
// styles:
onButtonStylesChange, usesButtonLayout, } from '@reusable-ui/button'; // a button component
// internals:
import { 
// elements:
dummyElm, inputElm, checkElm, labelElm, } from './elements.js';
import { 
// configs:
checks, cssCheckConfig, } from './config.js';
// styles:
export const onCheckStylesChange = watchChanges(onEditableActionControlStylesChange, onButtonStylesChange, cssCheckConfig.onChange);
export const usesCheckLayout = memoizeStyle(() => {
    // dependencies:
    // features:
    const { foregroundVars } = usesForeground();
    const { paddingVars } = usesPadding();
    // states:
    const { checkableVars } = usesCheckable();
    return style({
        // layouts:
        ...usesEditableActionControlLayout(),
        ...style({
            // layouts:
            display: 'inline-flex',
            flexDirection: 'row',
            justifyContent: 'center',
            alignItems: 'center',
            flexWrap: 'wrap',
            // positions:
            verticalAlign: 'baseline',
            // children:
            ...children(dummyElm, {
                ...fillTextLineHeightLayout(),
            }),
            ...children(inputElm, {
                // layouts:
                ...usesEditableActionControlLayout(),
                ...style({
                    // layouts:
                    display: 'inline-block',
                    // sizes:
                    boxSizing: 'border-box',
                    // the size is exactly the same as current font size:
                    inlineSize: '1em',
                    blockSize: '1em',
                    flex: [[0, 0, 'auto']],
                    // accessibilities:
                    pointerEvents: 'none',
                    // borders:
                    overflow: 'hidden',
                    // animations:
                    filter: 'none',
                    anim: 'inherit',
                    // spacings:
                    [paddingVars.paddingInline]: '0px',
                    [paddingVars.paddingBlock]: '0px',
                    ...ifNotLastChild({
                        // spacing between input & label:
                        marginInlineEnd: checks.spacing, // we cannot place a `gap` on the parent flex because the existance of <dummyElm>
                    }),
                    // children:
                    ...children(checkElm, {
                        // layouts:
                        // check indicator:
                        ...usesIconImage({
                            image: checks.indicator,
                            color: foregroundVars.foreg,
                        }),
                        ...style({
                            // layouts:
                            content: '""',
                            display: 'block',
                            // sizes:
                            // fills the entire parent:
                            boxSizing: 'border-box',
                            blockSize: '100%',
                            // animations:
                            filter: checkableVars.filter,
                            transform: checkableVars.transform,
                            anim: checkableVars.anim,
                        }),
                    }),
                    // customize:
                    ...usesCssProps(checks), // apply config's cssProps
                }),
            }),
            ...children(labelElm, {
                // layouts:
                display: 'inline',
                // positions:
                verticalAlign: 'baseline',
                // sizes:
                flex: [[1, 1, 0]],
                // customize:
                ...usesCssProps(usesPrefixedProps(checks, 'label')), // apply config's cssProps starting with label***
            }),
        }),
    });
}, onCheckStylesChange);
export const usesCheckVariants = memoizeStyle(() => {
    // dependencies:
    // features:
    const { foregroundVars } = usesForeground();
    const { borderVars } = usesBorder();
    // variants:
    const { resizableRule } = usesResizable(checks);
    const { mildableVars } = usesMildable();
    return style({
        // variants:
        /* write specific checkStyle first, so it can be overriden by `.nude`, `.mild`, `.outlined`, etc */
        ...variants([
            rule(['.button', '.toggleButton'], {
                // layouts:
                ...usesButtonLayout(),
                ...style({
                    // children:
                    // hide the <dummy> & <Check>'s indicator:
                    ...children([dummyElm, inputElm], {
                        // layouts:
                        display: 'none',
                    }),
                    // customize:
                    ...usesCssProps(usesPrefixedProps(checks, 'button')),
                    // overwrites propName = {button}propName:
                    ...overwriteProps(checks, usesPrefixedProps(checks, 'button')),
                }),
            }),
            rule('.toggleButton', {
                // customize:
                ...usesCssProps(usesPrefixedProps(checks, 'toggleButton')),
                // overwrites propName = {toggleButton}propName:
                ...overwriteProps(checks, usesPrefixedProps(checks, 'toggleButton')),
            }),
            rule('.switch', {
                // children:
                ...children(inputElm, {
                    // sizes:
                    aspectRatio: '2 / 1',
                    inlineSize: 'auto',
                    // borders:
                    // circle corners on top:
                    [borderVars.borderStartStartRadius]: borderRadiuses.pill,
                    [borderVars.borderStartEndRadius]: borderRadiuses.pill,
                    // circle corners on bottom:
                    [borderVars.borderEndStartRadius]: borderRadiuses.pill,
                    [borderVars.borderEndEndRadius]: borderRadiuses.pill,
                    // customize:
                    ...usesCssProps(usesPrefixedProps(checks, 'switch')), // apply config's cssProps starting with switch***
                }),
                // overwrites propName = {switch}propName:
                ...overwriteProps(checks, usesPrefixedProps(checks, 'switch')),
            }),
        ], { specificityWeight: 1 }),
        ...usesEditableActionControlVariants(),
        ...resizableRule(),
        ...variants([
            ifNotNude({
                // children:
                ...children(inputElm, {
                    // borders:
                    [borderVars.borderColor]: foregroundVars.foreg,
                    // animations:
                    boxShadow: ['none', '!important'], // remove double focus indicator animation to the wrapper
                }),
            }),
            ifNude({
                // foregrounds:
                foreg: [mildableVars.foregFn, '!important'], // no valid/invalid animation
            }),
        ], { specificityWeight: 2 }),
    });
}, onCheckStylesChange);
export const usesCheckStates = memoizeStyle(() => {
    // dependencies:
    // states:
    const { focusableVars } = usesFocusable();
    const { checkableRule } = usesCheckable(checks);
    const { activeAsClickRule } = usesActiveAsClick();
    return style({
        // states:
        ...usesEditableActionControlStates(),
        ...checkableRule(),
        ...variants([
            rule('.toggleButton', {
                // states:
                ...activeAsClickRule(),
            }),
        ], { specificityWeight: 1 }),
        // layouts:
        ...style({
            // children:
            ...children(inputElm, {
                ...vars({
                    // copy focus effect from parent:
                    [focusableVars.boxShadow]: 'inherit',
                    [focusableVars.anim]: 'inherit',
                }),
            }),
        }),
    });
}, onCheckStylesChange);
export default memoizeStyle(() => style({
    // layouts:
    ...usesCheckLayout(),
    // variants:
    ...usesCheckVariants(),
    // states:
    ...usesCheckStates(),
}), onCheckStylesChange);
